@@ -1,21 +1,34 @@
 
-/* $Id: php_pdf.h,v 1.18.20.2 2013/04/05 14:25:58 rjs Exp $ */
+/* $Id: php_pdf.h,v 1.18.20.6.2.1 2016/12/23 12:53:25 rjs Exp $ */
 /* Derived from:
     Id: php_pdf.h,v 1.22 2001/11/30 04:46:35 sniper Exp */
 
 #ifndef PHP_PDF_H
 #define PHP_PDF_H
 
-#if HAVE_PDFLIB
+extern zend_module_entry PDFlib_module_entry;
+#define phpext_pdf_ptr &PDFlib_module_entry
 
-#include <pdflib.h>
+#ifdef PHP_WIN32
+#   define PHP_PDFLIB_API __declspec(dllexport)
+#elif defined(__GNUC__) && __GNUC__ >= 4
+#   define PHP_PDFLIB_API __attribute__ ((visibility("default")))
+#else
+#   define PHP_PDFLIB_API
+#endif
 
-extern zend_module_entry pdf_module_entry;
-#define pdf_module_ptr &pdf_module_entry
+#ifdef ZTS
+#include "TSRM.h"
+#endif
 
-PHP_MINFO_FUNCTION(pdf);
-PHP_MINIT_FUNCTION(pdf);
-PHP_MSHUTDOWN_FUNCTION(pdf);
+#ifndef PHP_FE_END
+#define PHP_FE_END {NULL, NULL, NULL}
+#endif
+
+
+PHP_MINFO_FUNCTION(PDFlib);
+PHP_MINIT_FUNCTION(PDFlib);
+PHP_MSHUTDOWN_FUNCTION(PDFlib);
 
 #define _WRAP_FUNCTION_ENTRY3
 #include "php_wrapped.c"
@@ -35,16 +48,10 @@ PHP_FUNCTION(pdf_utf32_to_utf8);
 PHP_FUNCTION(pdf_utf8_to_utf32);
 PHP_FUNCTION(pdf_utf16_to_utf32);
 
-#ifdef ZTS
-#define PDFG(v) TSRMG(pdf_globals_id, php_pdf_globals *, v)
-#else
-#define PDFG(v) (pdf_globals.v)
+#if PHP_MAJOR_VERSION >= 7
+#if defined(ZTS) && defined(COMPILE_DL_PDFLIB)
+ZEND_TSRMLS_CACHE_EXTERN();
 #endif
+#endif /* PHP_MAJOR_VERSION >= 7 */
 
-
-#else
-#define pdf_module_ptr NULL
 #endif
-
-#define phpext_pdf_ptr pdf_module_ptr
-#endif /* PHP_PDF_H */
